package info.aspirecn.cloud.yysj.cloudyysjlogin.controller.exception;

import com.auth0.jwt.exceptions.JWTVerificationException;
import com.auth0.jwt.exceptions.TokenExpiredException;
import info.aspirecn.cloud.yysj.cloudyysjlogin.controller.YysjUserControllder;
import info.aspirecn.cloud.yysj.cloudyysjlogin.entity.response.ResultResponse;
import info.aspirecn.cloud.yysj.cloudyysjlogin.enums.ResultEnum;
import info.aspirecn.cloud.yysj.cloudyysjlogin.exceptions.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.client.HttpClientErrorException;
import javax.servlet.http.HttpServletResponse;
import java.net.SocketTimeoutException;

@RestControllerAdvice(basePackageClasses = YysjUserControllder.class)
@Slf4j
public class GlobalExceptionHandle {
    /**
     * 自定义异常处理器
     *
     * @param response 响应对象
     * @return ResponseInfo
     */
    @ExceptionHandler({RuntimeException.class})
    public ResultResponse runtimeExceptionHandler(RuntimeException e, HttpServletResponse response) {
        // 设置响应对象
        ResultResponse resultResponse = new ResultResponse();
        response.setStatus(HttpServletResponse.SC_OK);
        if (e instanceof UserNotExistException) {
            //用户名或密码错误
            resultResponse.setCode(ResultEnum.YYSJ_NO_USER.getStatus())
                          .setError(ResultEnum.YYSJ_NO_USER);
        } else if (e instanceof SystemBusyException) {
            resultResponse.setCode(ResultEnum.YYSJ_SYSTEM_BUSY.getStatus())
                    .setError(ResultEnum.YYSJ_SYSTEM_BUSY);
        } else if ((e instanceof TokenExpiredException) || (e instanceof TokenTimeoutException)) {
            resultResponse.setCode(ResultEnum.YYSJ_REQUEST_TOKEN_OUTTIME.getStatus())
                    .setError(ResultEnum.YYSJ_REQUEST_TOKEN_OUTTIME);
        } else if (e instanceof JWTVerificationException) {
            resultResponse.setCode(ResultEnum.YYSJ_REQUEST_TOKEN_ERROR.getStatus())
                    .setError(ResultEnum.YYSJ_REQUEST_TOKEN_ERROR);
        } else if (e instanceof LoginAgainException) {
            resultResponse.setCode(ResultEnum.YYSJ_LOGIN_AGAIN.getStatus())
                    .setError(ResultEnum.YYSJ_LOGIN_AGAIN);
        } else if (e instanceof ParamaterException) {
            resultResponse.setCode(ResultEnum.YYSJ_REQUEST_PARAMETER_ERROR.getStatus())
                    .setError(ResultEnum.YYSJ_REQUEST_PARAMETER_ERROR);
        }else {
            resultResponse.setCode(ResultEnum.YYSJ_SYSTEM_ERROR.getStatus())
                    .setError(ResultEnum.YYSJ_SYSTEM_ERROR);
        }
        log.error("", e);
        return resultResponse;
    }
    /**
     * 连接超时异常处理
     *
     * @param response 响应对象
     * @return ResponseInfo
     */
    @ExceptionHandler({SocketTimeoutException.class, HttpClientErrorException.class})
    public ResultResponse jWTDecodeExceptionHandler(Exception e, HttpServletResponse response) {
        // 设置响应对象
        ResultResponse resultResponse = new ResultResponse();
        resultResponse.setCode(ResultEnum.YYSJ_SYSTEM_ERROR.getStatus()).
                setError(ResultEnum.YYSJ_SYSTEM_ERROR);
        return resultResponse;
    }

}
