package info.aspirecn.cloud.yysj.cloudyysjlogin.service.impl;

import info.aspirecn.cloud.yysj.cloudyysjlogin.entity.response.ResultResponse;
import info.aspirecn.cloud.yysj.cloudyysjlogin.enums.ResultEnum;
import info.aspirecn.cloud.yysj.cloudyysjlogin.exceptions.LoginAgainException;
import info.aspirecn.cloud.yysj.cloudyysjlogin.exceptions.TokenTimeoutException;
import info.aspirecn.cloud.yysj.cloudyysjlogin.service.CheckTokenService;
import info.aspirecn.cloud.yysj.cloudyysjlogin.utils.JwtUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service
public class CheckTokenServiceImpl implements CheckTokenService {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Value(("${yysj.redis.prefix}"))
    private String prefix;

    @Override
    public String checkToken(String token) {
        ResultResponse resultResponse = new ResultResponse();
        // 验证 Token 是否正确
        Map map = JwtUtils.verifyToken(token);
        String uid = String.valueOf(map.get("userid"));
        //如果redis里没有缓存则提示重新登陆
        String key = prefix + uid;
        Object object = redisTemplate.opsForValue().get(key);
        if(object==null){
            throw new LoginAgainException();
        }
        if(!String.valueOf(object).equals(token)){
            throw new TokenTimeoutException("请求token错误");
        }
        //更新redis缓存时间
        redisTemplate.expire(key, 30, TimeUnit.MINUTES);
        resultResponse.setSuccess(ResultEnum.YYSJ_SUCCESS, uid);
        return uid;
    }

}
