package info.aspirecn.cloud.yysj.cloudyysjlogin.utils;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.auth0.jwt.interfaces.JWTVerifier;
import info.aspirecn.cloud.yysj.cloudyysjlogin.exceptions.SystemBusyException;

import org.apache.commons.codec.binary.Base64;
import org.omg.CORBA.SystemException;

import java.security.KeyFactory;
import java.security.interfaces.RSAPrivateKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.crypto.Cipher;

public class JwtUtils {

    private JwtUtils() {
    }

    /**
     * 签名的私钥
     */
    private static final String SECRET = "9a96349e2345385785e804e0f4254dee";
    /**
     * 签发人
     */
    private static final String ISSUER = "SYS_USER";
    /**RSA*/
    private static final String RSA = "RSA";
    /**utf-8*/
    private static final String ENCODING_UTF8 = "UTF-8";

    /**
     * 生成token
     *
     * @param claims 加密内容
     * @param time   过期时间，单位(s)
     * @return Token字符串
     */
    public static String createToken(Map<String, String> claims, long time) {
        try {
            // 设置过期时间
            Date expireDate = new Date();
            expireDate.setTime(expireDate.getTime() + time * 1000);
            //使用HMAC256进行加密
            Algorithm algorithm = Algorithm.HMAC256(SECRET);
            //创建jwt
            JWTCreator.Builder builder = JWT.create()
                    .withIssuer(ISSUER).
                            withExpiresAt(expireDate);
            //传入参数
            claims.forEach(builder::withClaim);
            //签名加密
            return builder.sign(algorithm);
        } catch (IllegalArgumentException e) {
            throw new SystemBusyException("创建token异常");
        }
    }

    /**
     * 解密jwt
     *
     * @param token 加密串
     * @return jwt 负载信息
     */
    public static Map<String, String> verifyToken(String token) {
        Algorithm algorithm = null;
        //使用HMAC256进行加密
        algorithm = Algorithm.HMAC256(SECRET);
        //解密
        JWTVerifier verifier = JWT.require(algorithm).withIssuer(ISSUER).build();
        DecodedJWT jwt = verifier.verify(token);
        Map<String, Claim> map = jwt.getClaims();
        Map<String, String> resultMap = new HashMap<>();
        map.forEach((k, v) -> resultMap.put(k, v.asString()));
        return resultMap;
    }
    
    /** 
	 * RSA私钥解密
	 *  
	 * @param str 
	 *            加密字符串
	 * @param privateKey 
	 *            私钥 
	 * @return 铭文
	 * @throws Exception 
	 *             解密过程中的异常信息 
	 */  
	public static String rsaDecrypt(String str, String privateKey) throws Exception{
		//64位解码加密后的字符串
		byte[] inputByte = Base64.decodeBase64(str.getBytes(ENCODING_UTF8));
		//base64编码的私钥
		byte[] decoded = Base64.decodeBase64(privateKey);  
        RSAPrivateKey priKey = (RSAPrivateKey) KeyFactory.getInstance(RSA).generatePrivate(new PKCS8EncodedKeySpec(decoded));  
		//RSA解密
		Cipher cipher = Cipher.getInstance(RSA);
		cipher.init(Cipher.DECRYPT_MODE, priKey);
		String outStr = new String(cipher.doFinal(inputByte));
		return outStr;
	}

}